@@ -22,7 +22,21 @@
 
 #include "gdbmdefs.h"
 
-
+int
+gdbm_bucket_element_valid_p (GDBM_FILE dbf, int elem_loc)
+{
+  return
+    elem_loc < dbf->header->bucket_elems
+    && dbf->bucket->h_table[elem_loc].hash_value != -1
+    && dbf->bucket->h_table[elem_loc].key_size >= 0
+    && off_t_sum_ok (dbf->bucket->h_table[elem_loc].data_pointer,
+		     dbf->bucket->h_table[elem_loc].key_size)
+    && dbf->bucket->h_table[elem_loc].data_size >= 0
+    && off_t_sum_ok (dbf->bucket->h_table[elem_loc].data_pointer
+		     + dbf->bucket->h_table[elem_loc].key_size,
+		     dbf->bucket->h_table[elem_loc].data_size);
+}
+  
 /* Read the data found in bucket entry ELEM_LOC in file DBF and
    return a pointer to it.  Also, cache the read value. */
 
@@ -32,42 +46,66 @@ _gdbm_read_entry (GDBM_FILE dbf, int elem_loc)
   int rc;
   int key_size;
   int data_size;
+  size_t dsize;
   off_t file_pos;
   data_cache_elem *data_ca;
-  
+
   /* Is it already in the cache? */
   if (dbf->cache_entry->ca_data.elem_loc == elem_loc)
     return dbf->cache_entry->ca_data.dptr;
 
+  if (!gdbm_bucket_element_valid_p (dbf, elem_loc))
+    {
+      GDBM_SET_ERRNO (dbf, GDBM_BAD_HASH_TABLE, TRUE);
+      return NULL;
+    }
+  
   /* Set sizes and pointers. */
   key_size = dbf->bucket->h_table[elem_loc].key_size;
   data_size = dbf->bucket->h_table[elem_loc].data_size;
+  dsize = key_size + data_size;
   data_ca = &dbf->cache_entry->ca_data;
-  
+
   /* Set up the cache. */
-  if (data_ca->dptr != NULL) free (data_ca->dptr);
   data_ca->key_size = key_size;
   data_ca->data_size = data_size;
   data_ca->elem_loc = elem_loc;
   data_ca->hash_val = dbf->bucket->h_table[elem_loc].hash_value;
 
-  if (GDBM_DEBUG_HOOK ("_gdbm_read_entry:malloc-failure"))
-    data_ca->dptr = NULL;
-  else if (key_size + data_size == 0)
-    data_ca->dptr = (char *) malloc (1);
-  else
-    data_ca->dptr = (char *) malloc (key_size + data_size);
-  if (data_ca->dptr == NULL)
+  if (dsize <= data_ca->dsize)
     {
-      GDBM_SET_ERRNO2 (dbf, GDBM_MALLOC_ERROR, FALSE, GDBM_DEBUG_LOOKUP);
-      _gdbm_fatal (dbf, _("malloc error"));
-      return NULL;
+      if (data_ca->dsize == 0)
+	{
+	  data_ca->dptr = malloc (1);
+	  if (data_ca->dptr)
+	    data_ca->dsize = 1;
+	  else
+	    {
+	      GDBM_SET_ERRNO2 (dbf, GDBM_MALLOC_ERROR, FALSE, GDBM_DEBUG_LOOKUP);
+	      _gdbm_fatal (dbf, _("malloc error"));
+	      return NULL;
+	    }
+	}
+    }
+  else
+    {
+      char *p = realloc (data_ca->dptr, dsize);
+      if (p)
+	{
+	  data_ca->dptr = p;
+	  data_ca->dsize = dsize;
+	}
+      else
+	{
+	  GDBM_SET_ERRNO2 (dbf, GDBM_MALLOC_ERROR, FALSE, GDBM_DEBUG_LOOKUP);
+	  _gdbm_fatal (dbf, _("malloc error"));
+	  return NULL;
+	}
     }
 
   /* Read into the cache. */
-  file_pos = GDBM_DEBUG_OVERRIDE ("_gdbm_read_entry:lseek-failure",
-	      __lseek (dbf, dbf->bucket->h_table[elem_loc].data_pointer, 
-		       SEEK_SET));
+  file_pos = gdbm_file_seek (dbf, dbf->bucket->h_table[elem_loc].data_pointer, 
+		      SEEK_SET);
   if (file_pos != dbf->bucket->h_table[elem_loc].data_pointer)
     {
       GDBM_SET_ERRNO2 (dbf, GDBM_FILE_SEEK_ERROR, TRUE, GDBM_DEBUG_LOOKUP);
@@ -75,15 +113,14 @@ _gdbm_read_entry (GDBM_FILE dbf, int elem_loc)
       return NULL;
     }
   
-  rc = GDBM_DEBUG_OVERRIDE ("_gdbm_read_entry:read-failure",
-	    _gdbm_full_read (dbf, data_ca->dptr, key_size+data_size));
+  rc = _gdbm_full_read (dbf, data_ca->dptr, key_size+data_size);
   if (rc)
     {
       GDBM_DEBUG (GDBM_DEBUG_ERR|GDBM_DEBUG_LOOKUP|GDBM_DEBUG_READ,
 		  "%s: error reading entry: %s",
 		  dbf->name, gdbm_db_strerror (dbf));
       dbf->need_recovery = TRUE;
-      _gdbm_fatal (dbf, gdbm_strerror (rc));
+      _gdbm_fatal (dbf, gdbm_db_strerror (dbf));
       return NULL;
     }
   
