@@ -1,8 +1,8 @@
 /* gdbmcount.c - get number of items in a gdbm file. */
 
 /* This file is part of GDBM, the GNU data base manager.
-   Copyright (C) 1993, 1994, 2007, 2011, 2013,
-   2016 Free Software Foundation, Inc.
+   Copyright (C) 1993-1994, 2007, 2011, 2013, 2016-2018 Free Software
+   Foundation, Inc.
 
    GDBM is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
