@@ -74,9 +74,9 @@ gdbm_avail_block_valid_p (avail_block const *av)
   return (av->size > 1 && av->count >= 0 && av->count <= av->size);
 }
 
-/* Return true if both AV and the size of AV->av_table are valid */
-extern int gdbm_avail_table_valid_p (GDBM_FILE dbf,
-				     avail_elem const *av, int count);
+/* Return true if both AV and the size of AV->av_table are valid.
+   See comment to this function in gdbmopen.c */
+extern int gdbm_avail_table_valid_p (GDBM_FILE dbf, avail_elem *av, int count);
 
 /* The dbm file header keeps track of the current location of the hash
    directory and the free space in the file.  */
