@@ -99,7 +99,7 @@ gdbm_avail_table_valid_p (GDBM_FILE dbf, avail_elem *av, int count)
 
   if (needs_sorting && dbf->read_write)
     {
-      GDBM_DEBUG (GDBM_DEBUG_ERR, "restoring sort order");
+      GDBM_DEBUG (GDBM_DEBUG_ERR, "%s", "restoring sort order");
       qsort (av, count, sizeof av[0], avail_comp);
     }
   
@@ -136,6 +136,7 @@ static int
 validate_header (gdbm_file_header const *hdr, struct stat const *st)
 {
   int dir_size, dir_bits;
+  int result = GDBM_NO_ERROR;
   
   /* Is the magic number good? */
   if (hdr->header_magic != GDBM_MAGIC)
@@ -168,9 +169,14 @@ validate_header (gdbm_file_header const *hdr, struct stat const *st)
       return GDBM_BLOCK_SIZE_ERROR;
     }
 
-  if (hdr->next_block != st->st_size)
-    /* FIXME: Should return GDBM_NEED_RECOVERY instead? */
-    return GDBM_BAD_HEADER;
+  /* Technically speaking, the condition below should read
+         hdr->next_block != st->st_size
+     However, gdbm versions prior to commit 4e819c98 could leave
+     hdr->next_block pointing beyond current end of file. To ensure
+     backward compatibility with these versions, the condition has been
+     slackened to this: */
+  if (hdr->next_block < st->st_size)
+    result = GDBM_NEED_RECOVERY;
 
   /* Make sure dir and dir + dir_size fall within the file boundary */
   if (!(hdr->dir > 0
@@ -196,9 +202,35 @@ validate_header (gdbm_file_header const *hdr, struct stat const *st)
       != hdr->avail.size)
     return GDBM_BAD_HEADER;    
   
-  return 0;
+  return result;
 }
-  
+
+int
+_gdbm_validate_header (GDBM_FILE dbf)
+{
+  struct stat file_stat;
+    
+  if (fstat (dbf->desc, &file_stat))
+    return GDBM_FILE_STAT_ERROR;
+
+  return validate_header (dbf->header, &file_stat);
+}
+
+/* Do we have ftruncate? */
+static inline int
+_gdbm_ftruncate (GDBM_FILE dbf)
+{
+#if HAVE_FTRUNCATE
+  return ftruncate (dbf->desc, 0);
+#else
+  int fd;
+  fd = open (dbf->name, O_RDWR|O_TRUNC, mode);
+  if (fd == -1)
+    return -1;
+  return close (fd);
+#endif
+}
+
 GDBM_FILE 
 gdbm_fd_open (int fd, const char *file_name, int block_size,
 	      int flags, void (*fatal_func) (const char *))
@@ -246,7 +278,7 @@ gdbm_fd_open (int fd, const char *file_name, int block_size,
   dbf->mapped_size = 0;
   dbf->mapped_pos = 0;
   dbf->mapped_off = 0;
-  
+
   /* Save name of file. */
   dbf->name = strdup (file_name);
   if (dbf->name == NULL)
@@ -320,14 +352,22 @@ gdbm_fd_open (int fd, const char *file_name, int block_size,
      now time to truncate the file. */
   if ((flags & GDBM_OPENMASK) == GDBM_NEWDB && file_stat.st_size != 0)
     {
-      TRUNCATE (dbf);
-      if (fstat (dbf->desc, &file_stat))
+      if (_gdbm_ftruncate (dbf))
+	{
+	  GDBM_SET_ERRNO2 (dbf, GDBM_FILE_TRUNCATE_ERROR, FALSE,
+			   GDBM_DEBUG_OPEN);
+	}
+      else if (fstat (dbf->desc, &file_stat))
+	{
+	  GDBM_SET_ERRNO2 (dbf, GDBM_FILE_STAT_ERROR, FALSE, GDBM_DEBUG_OPEN);
+	}
+
+      if (gdbm_last_errno (dbf))
 	{
 	  if (flags & GDBM_CLOERROR)
 	    close (dbf->desc);
 	  free (dbf->name);
 	  free (dbf);
-	  GDBM_SET_ERRNO2 (NULL, GDBM_FILE_STAT_ERROR, FALSE, GDBM_DEBUG_OPEN);
 	  return NULL;
 	}
     }
@@ -498,7 +538,11 @@ gdbm_fd_open (int fd, const char *file_name, int block_size,
 
       /* Is the header valid? */
       rc = validate_header (&partial_header, &file_stat);
-      if (rc != GDBM_NO_ERROR)
+      if (rc == GDBM_NEED_RECOVERY)
+	{
+	  dbf->need_recovery = 1;
+	}
+      else if (rc != GDBM_NO_ERROR)
 	{
 	  if (!(flags & GDBM_CLOERROR))
 	    dbf->desc = -1;
@@ -604,7 +648,8 @@ gdbm_fd_open (int fd, const char *file_name, int block_size,
   dbf->bucket_changed = FALSE;
   dbf->second_changed = FALSE;
 
-  GDBM_DEBUG (GDBM_DEBUG_ALL, "%s: opened successfully", dbf->name);
+  GDBM_DEBUG (GDBM_DEBUG_ALL, "%s: opened %s", dbf->name,
+	      dbf->need_recovery ? "for recovery" : "successfully");
 
   /* Everything is fine, return the pointer to the file
      information structure.  */
@@ -641,20 +686,23 @@ gdbm_open (const char *file, int block_size, int flags, int mode,
 
   switch (flags & GDBM_OPENMASK)
     {
-      case GDBM_READER:
-	fbits = O_RDONLY;
-	break;
+    case GDBM_READER:
+      fbits = O_RDONLY;
+      break;
 
-      case GDBM_WRITER:
-	fbits = O_RDWR;
-	break;
+    case GDBM_WRITER:
+      fbits = O_RDWR;
+      break;
 
-      case GDBM_NEWDB:
-	fbits = O_RDWR|O_CREAT;
-	break;
+    case GDBM_WRCREAT:
+    case GDBM_NEWDB:
+      fbits = O_RDWR|O_CREAT;
+      break;
 
-      default:
-	fbits = O_RDWR|O_CREAT;
+    default:
+      errno = EINVAL;
+      GDBM_SET_ERRNO2 (NULL, GDBM_FILE_OPEN_ERROR, FALSE, GDBM_DEBUG_OPEN);
+      return NULL;
     }
   if (flags & GDBM_CLOEXEC)
     fbits |= O_CLOEXEC;
