@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include "gdbm.h"
 #include "progname.h"
 
@@ -115,6 +116,11 @@ main (int argc, char **argv)
       exit (1);
     }
   
-  gdbm_close (dbf);
+  if (gdbm_close (dbf))
+    {
+      fprintf (stderr, "gdbm_close: %s; %s\n", gdbm_strerror (gdbm_errno),
+	       strerror (errno));
+      exit (3);
+    }
   exit (0);
 }
