@@ -19,6 +19,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <errno.h>
 #include "gdbm.h"
 #include "progname.h"
 #include <assert.h>
@@ -119,6 +120,11 @@ main (int argc, char **argv)
 
   rc = gdbm_recover (dbf, &rcvr, rcvr_flags);
 
-  gdbm_close (dbf);
+  if (gdbm_close (dbf))
+    {
+      fprintf (stderr, "gdbm_close: %s; %s\n", gdbm_strerror (gdbm_errno),
+	       strerror (errno));
+      rc = 3;
+    }
   exit (rc);
 }
